import json
from os import devnull, environ
from subprocess import Popen, PIPE
from n_utils.aws_infra_util import load_parameters

ARR_START = "[".encode()
OBJ_START = "{".encode()

def az_login():
    proc = Popen(
        ["az", "login"],
        stdout=PIPE,
        stderr=PIPE,
    )
    output, err = proc.communicate()
    if proc.returncode != 0:
        return []
    return json.loads(output[output.find(ARR_START):])

def az_logout():
    proc = Popen(
        ["az", "logout"],
        stdout=PIPE,
        stderr=PIPE,
    )
    output, err = proc.communicate()
    return proc.returncode == 0

def az_account_show():
    proc = Popen(
        ["az", "account", "show"],
        stdout=PIPE,
        stderr=PIPE,
    )
    output, _err = proc.communicate()
    if proc.returncode != 0:
        return {}
    return json.loads(output[output.find(OBJ_START):])

def az_account_list():
    proc = Popen(
        ["az", "account", "list"],
        stdout=PIPE,
        stderr=PIPE,
    )
    output, err = proc.communicate()
    if proc.returncode != 0:
        return []
    return json.loads(output[output.find(ARR_START):])

def az_find_subscription_id(name, account_list):
    for subscription in account_list:
        if subscription["name"] == name or subscription["id"] == name:
            return subscription["id"]
    return None

def az_set_subscription(subscription_id):
    proc = Popen(
        ["az", "account", "set", "--subscription", subscription_id],
        stdout=PIPE,
        stderr=PIPE,
    )
    output, err = proc.communicate()
    return proc.returncode == 0

def az_select_subscription(name):
    curr_account = az_account_show()
    subscription_id = None
    if "name" in curr_account and "id" in curr_account:
        if curr_account["name"] == name or curr_account["id"] == name:
            return curr_account["id"]
        else:
            subscription_id = az_find_subscription_id(name, az_account_list())
    if not subscription_id:
        az_logout()
        subscription_id = az_find_subscription_id(name, az_login())

    if subscription_id:
        az_set_subscription(subscription_id)
    return subscription_id

def ensure_group(location, group_name):
    proc = Popen(
        ["az", "group", "list", "--query", "[?name=='" + group_name + "']"],
        stdout=PIPE,
        stderr=PIPE,
    )
    output, err = proc.communicate()
    groups = json.loads(output[output.find(ARR_START):])
    if len(groups) == 0:
        proc = Popen(
            ["az", "group", "create", "--name", group_name, "--location", location],
            stdout=PIPE,
            stderr=PIPE,
        )
        output, err = proc.communicate()
        if proc.returncode == 0:
            groups = [json.loads(output[output.find(OBJ_START):])]
        else:
            groups = [{}]
    return groups[0]

def ensure_management_group(group_name):
    proc = Popen(
        ["az", "account", "management-group", "list", "--query", "[?name=='" + group_name + "']"],
        stdout=PIPE,
        stderr=PIPE,
    )
    output, err = proc.communicate()
    groups = json.loads(output[output.find(ARR_START):])
    if len(groups) == 0:
        proc = Popen(
            ["az", "account", "management-group", "create", "--name", group_name],
            stdout=PIPE,
            stderr=PIPE,
        )
        output, err = proc.communicate()
        if proc.returncode == 0:
            groups = [json.loads(output[output.find(OBJ_START):])]
        else:
            groups = [{}]
    

def delete_group(group_name):
    proc = Popen(
        ["az", "group", "delete", "-y", "--name", group_name],
        stdout=PIPE,
        stderr=PIPE,
    )
    output, err = proc.communicate()
    return proc.returncode == 0

def resolve_location():
    if "AZURE_LOCATION" in environ and environ["AZURE_LOCATION"] and environ["AZURE_LOCATION"] != "default":
        return environ["AZURE_LOCATION"]
    else:
        parameters = load_parameters()
        if "AZURE_LOCATION" in parameters and parameters["AZURE_LOCATION"] and parameters["AZURE_LOCATION"] != "default":
            return parameters["AZURE_LOCATION"]
        else:
            proc = Popen(
                ["az", "configure", "--list-defaults"],
                stdout=PIPE,
                stderr=PIPE,
            )
            default_location = None
            output, err = proc.communicate()
            if proc.returncode == 0 and output:
                confs = json.loads(output)
                for conf in confs:
                    if "name" in conf and conf["name"] == "location" and \
                       "value" in conf and conf["value"]:
                        default_location = conf["value"]
            if default_location:
                return default_location
            else:
                return "northeurope"



from locale import getpreferredencoding
from os import linesep, environ, devnull
from sys import argv
from n_utils.profile_util import enable_profile
from subprocess import Popen, PIPE

SYS_ENCODING = getpreferredencoding()

def _var_name(suffix):
    return "ndt.profile." + suffix

def _branch_var_name(current_branch, suffix):
    return "ndt.profile." + current_branch + "." + suffix

def _check_enable_profile(vars, current_branch, profile_type):
    if _branch_var_name(current_branch, profile_type) in vars:
        enable_profile(profile_type, vars[_branch_var_name(current_branch, profile_type)])
    elif _var_name(profile_type) in vars:
        enable_profile(profile_type, vars[_var_name(profile_type)])

def load_project_env():
    """ Print parameters set by git config variables to setup project environment with region and aws credentials
    """
    proc = Popen(["git", "config", "--list", "--local"], stdout=PIPE, stderr=open(devnull, 'w'))
    proc2 = Popen(["git", "rev-parse", "--abbrev-ref", "HEAD"], stdout=PIPE, stderr=open(devnull, 'w'))
    stdout, _ = proc2.communicate()
    current_branch = "master"
    if proc2.returncode == 0:
        current_branch = stdout.decode(SYS_ENCODING).strip()
    out, _ = proc.communicate()
    if proc.returncode:
        return
    vars = {}
    for line in out.decode(SYS_ENCODING).split("\n"):
        if line:
            next = line.split("=", 1)
            vars[next[0]] = next[1]
    do_print = False
    ret = ""

    _check_enable_profile(vars, current_branch, "azure")
    _check_enable_profile(vars, current_branch, "adfs")
    _check_enable_profile(vars, current_branch, "lastpass")
    _check_enable_profile(vars, current_branch, "iam")
    _check_enable_profile(vars, current_branch, "ndt")
    _check_enable_profile(vars, current_branch, "azure-subscription")

    if "ndt.source." + current_branch + ".env" in vars:
        do_print = True
        ret = ret + ". " + vars["ndt.source." + current_branch + ".env"] + linesep
    elif "ndt.source.env" in vars:
        do_print = True
        ret = ret + ". " + vars["ndt.source.env"] + linesep

    if "ndt.aws." + current_branch + ".profile" in vars:
        do_print = True
        ret = ret + "export AWS_PROFILE=" + vars["ndt.aws." + current_branch + ".profile"] + \
            " AWS_DEFAULT_PROFILE=" + vars["ndt.aws." + current_branch + ".profile"] + linesep
    elif "ndt.aws.profile" in vars:
        do_print = True
        ret = ret + "export AWS_PROFILE=" + vars["ndt.aws.profile"] + \
            " AWS_DEFAULT_PROFILE=" + vars["ndt.aws.profile"] + linesep

    if "ndt.aws." + current_branch + ".region" in vars:
        do_print = True
        ret = ret + "export AWS_REGION=" + vars["ndt.aws." + current_branch + ".region"] + \
            " AWS_DEFAULT_REGION=" + vars["ndt.aws. " + current_branch + ".region"] + linesep
    elif "ndt.aws.region" in vars:
        do_print = True
        ret = ret + "export AWS_REGION=" + vars["ndt.aws.region"] + \
            " AWS_DEFAULT_REGION=" + vars["ndt.aws.region"] + linesep

    if do_print:
        print(ret.strip())


def ndt_register_complete():
    """Print out shell function and command to register ndt command completion
    """
    print("""_ndt_complete() {
    local IFS=$'\\013'
    local COMP_CUR="${COMP_WORDS[COMP_CWORD]}"
    local COMP_PREV="${COMP_WORDS[COMP_CWORD-1]}"
    local SUPPRESS_SPACE=0
    if compopt +o nospace 2> /dev/null; then
        SUPPRESS_SPACE=1
    fi
    COMPREPLY=( $(IFS="$IFS" \\
                  COMP_LINE="$COMP_LINE" \\
                  COMP_POINT="$COMP_POINT" \\
                  COMP_TYPE="$COMP_TYPE" \\
                  COMP_CUR="$COMP_CUR" \\
                  COMP_PREV="$COMP_PREV" \\
                  COMP_CWORD=$COMP_CWORD \\
                  _ARGCOMPLETE_COMP_WORDBREAKS="$COMP_WORDBREAKS" \\
                  _ARGCOMPLETE=1 \\
                  _ARGCOMPLETE_SUPPRESS_SPACE=$SUPPRESS_SPACE \\
                  "$1" 8>&1 9>&2 1>/dev/null 2>/dev/null) )
    if [[ $? != 0 ]]; then
        unset COMPREPLY
    elif [[ $SUPPRESS_SPACE == 1 ]] && [[ "$COMPREPLY" =~ [=/:]$ ]]; then
        compopt -o nospace
    fi
}
complete -o nospace -F _ndt_complete "ndt" """)
    if len(argv) > 1 and argv[1] == "--project-env":
        print("""_projectenv_hook() {
  local previous_exit_status=$?;
  eval "$(nameless-dt-load-project-env)";
  return $previous_exit_status;
};
if ! [[ "$PROMPT_COMMAND" =~ _projectenv_hook ]]; then
  PROMPT_COMMAND="_projectenv_hook;$PROMPT_COMMAND";
fi""")



import time
from threadlocal_aws.clients import route53


def upsert_record(dns_name, record_type, value, ttl=300, wait=True):
    zone_id, change = get_record_change(dns_name, record_type, value, hosted_zones(), ttl=ttl)
    request = route53().change_resource_record_sets(HostedZoneId=zone_id,
                                                    ChangeBatch={
                                                        'Changes': [change]
                                                    })['ChangeInfo']
    if wait:
        status = route53().get_change(Id=request['Id'])['ChangeInfo']['Status']
        while not status == 'INSYNC':
            print("Waiting for request to sync - " + str(request['Id']) + "(" + dns_name + ") status: " + status)
            time.sleep(2)
            status = route53().get_change(Id=request['Id'])['ChangeInfo']['Status']


def hosted_zones():
    pages = route53().get_paginator('list_hosted_zones')
    for page in pages.paginate():
        for hosted_zone in page.get('HostedZones', []):
            yield hosted_zone

def longest_matching_zone(alias, hosted_zones):
    ret = {'Name': ''}
    for zone in hosted_zones:
        if (alias + ".").endswith(zone['Name']) and len(zone['Name']) > len(ret['Name']):
            ret = zone
    return ret

def get_record_change(dns_name, record_type, value, hosted_zones, ttl=300):
    zone = longest_matching_zone(dns_name, hosted_zones)
    if zone:
        print(dns_name + " => " + "(" + record_type + ") '" + value + "' in " + zone['Name'])
        change = {
            'Action': 'UPSERT',
            'ResourceRecordSet': {
                'Name': dns_name,
                'Type': record_type,
                'TTL': ttl,
                'ResourceRecords': [{ 'Value': value }]}
        }
        return zone['Id'], change
